@@ -20,7 +20,7 @@
 # IN THE SOFTWARE.
 
 __author__ = 'Niklas Rosenstein <rosensteinniklas@gmail.com>'
-__version__ = '0.0.10'
+__version__ = '0.0.16'
 
 from nr.pylang.utils import NotSet
 import inspect
@@ -305,7 +305,7 @@ class Node(object):
 
     decoration = get_decoration(decoration_cls, self.context.decorations,
       self.decorations, self.datatype.get_decorations())
-    if not decoration and isinstance(decoration_cls, ClassDecoration):
+    if not decoration and issubclass(decoration_cls, ClassDecoration):
       if isinstance(self.datatype, StructType):
         decoration = decoration_cls.get(self.datatype.struct_cls)
       elif isinstance(self.datatype, PythonClassType):
@@ -523,5 +523,5 @@ def make_node(value, datatype, filename=None, decorations=None):
 from .collection import Collection
 from .datatypes import *
 from .decorations import *
-from .struct import Field, Struct, StructType, make_struct
+from .struct import Field, Struct, StructType, make_struct, cast_struct
 from .union import UnionType
