@@ -273,13 +273,12 @@ class FieldSpec(object):
     name will be assigned the name of their associated key. """
 
     fields = []
-    for key, value in six.iteritems(fields_dict):
-      if not isinstance(value, Field):
-        raise TypeError('expected Field, key {!r} got {}'.format(
-          key, type(value).__name__))
-      if not value.name:
-        value.name = key
-      fields.append(value)
+    for key, field in six.iteritems(fields_dict):
+      if not isinstance(field, Field):
+        field = Field(field)
+      if not field.name:
+        field.name = key
+      fields.append(field)
     return cls(fields)
 
   @classmethod
@@ -356,6 +355,15 @@ class FieldSpec(object):
   def __getitem__(self, name):
     return self._fields[name]
 
+  def __setitem__(self, name, field):  # type: (str, Field)
+    if not field.name:
+      field.name = name
+    self.update([field])
+
+  def __delitem__(self, name):
+    del self._fields[name]
+    self._update_cache()
+
   def __contains__(self, name):
     return name in self._fields
 
@@ -368,6 +376,9 @@ class FieldSpec(object):
   def __repr__(self):
     return 'FieldSpec({!r})'.format(list(self._fields.values()))
 
+  def __copy__(self):
+    return type(self)(self._fields.values())
+
   def keys(self):  # type: () - >Stream[str]
     return Stream(six.iterkeys(self._fields))
 
@@ -404,6 +415,9 @@ class FieldSpec(object):
   def prominent(self):  # type: () -> Iterable[Field]
     return iter(self._fields_prominent)
 
+  def copy(self):
+    return self.__copy__()
+
 
 class _StructMeta(type):
   """ Private. Meta class for #Struct. """
@@ -536,12 +550,16 @@ class Struct(six.with_metaclass(_StructMeta)):
     argcount = len(args) + len(kwargs)
     if argcount > len(self.__fields__):
       # TODO(nrosenstein): Include min number of args.
-      raise TypeError('expected at max {} arguments, got {}'
-                      .format(len(self.__fields__), argcount))
+      raise TypeError('{}() constructor expected at max {} arguments, got {}'
+                      .format(type(self).__name__, len(self.__fields__), argcount))
 
     # Check the type of all keyword arguments.
     for key, value in kwargs.items():
-      field = self.__fields__[key]
+      try:
+        field = self.__fields__[key]
+      except KeyError:
+        raise TypeError('{}() constructor received unexpected keyword argument {!r}'
+                        .format(type(self).__name__, key))
       if value is None and field.nullable:
         continue
       kwargs[key] = field.check_value(struct_name, value)
@@ -560,14 +578,26 @@ class Struct(six.with_metaclass(_StructMeta)):
 
     # Extract all fields.
     handled_keys = set()
+    missing_keys = set()
     for field in self.__fields__.values().sortby(lambda x: x.get_priority()):
       if field.name not in kwargs and field.name not in vars(self):
         if field.default is NotSet:
-          raise TypeError('{}: missing required argument "{}"'.format(
-            type(self).__name__, field.name))
+          missing_keys.add(field.name)
+          continue
         kwargs[field.name] = field.get_default_value()
       handled_keys.add(field.name)
 
+    # Give the class a chance to return the defaults for missing keys.
+    if missing_keys:
+      defaults = self._get_field_defaults(missing_keys)
+      defaults.update(kwargs)
+      kwargs = defaults
+
+    for key in missing_keys:
+      if key not in kwargs:
+        raise TypeError('{}: missing required argument "{}"'.format(
+          type(self).__name__, field.name))
+
     unhandled_keys = set(kwargs.keys()) - handled_keys
     if unhandled_keys:
       raise TypeError('unexpected keyword arguments: {!r}'.format(unhandled_keys))
@@ -599,6 +629,15 @@ class Struct(six.with_metaclass(_StructMeta)):
     attrs = ['{}={!r}'.format(f.name, getattr(self, f.name)) for f in fields]
     return '{}({})'.format(type(self).__name__, ', '.join(attrs))
 
+  @classmethod
+  def _get_field_defaults(cls, field_subset):  # type; (Set[str]) -> Dict[str, Any]
+    """
+    Class method that may be overwritten to provide default values when the Struct instance
+    is created that is not described in the #Field.default of the struct fields.
+    """
+
+    return {}
+
 
 def make_struct(name, fields, base=None, mixins=()):
   """
@@ -630,3 +669,14 @@ def make_struct(name, fields, base=None, mixins=()):
                       .format(type(key).__name__))
 
   return type(name, (base,) + mixins, {'__fields__': fields})
+
+
+def cast_struct(src, target_cls, **update):  # type: (Struct, Type[Struct], Any) -> Struct
+  """
+  Cast a struct instance from one to another class by taking all overlapping properties
+  from *src* and passing it to the *target_cls* constructor.
+  """
+
+  kwargs = {k: getattr(src, k) for k in src.__fields__ if k in target_cls.__fields__}
+  kwargs.update(update)
+  return target_cls(**kwargs)
